

const EXPORTED_SYMBOLS = ["JSONvsXML"];

var JSONvsXML = new function () {
    const NOT_NC_CHARS_RE= /([^0-9A-Z_a-z\u00C0-\u00D6\u00D8-\u00F6\u00F8-\u02FF\u0370-\u037D\u037F-\u1FFF\u200C-\u200D\u2070-\u218F\u2C00-\u2FEF\u3001-\uD7FF\uF900-\uFDCF\uFDF0-\uFFFD])/g;
    
    const NCName = new function () {
        const padNumber = function NCName__padNumber (str) {
            return Array(5 - str.length).join(0) + str;
        };
        
        const encodeSymbol = function NCName__encodeSymbol (symb) {
            return '.' + padNumber(symb.charCodeAt(0).toString(16).toUpperCase());
        };
        
        const decodeSymbol = function NCName__decodeSymbol (str) {
            return String.fromCharCode(parseInt(str.substring(1), 16));
        };
        
        const addPrefix = function NCName__addPrefix (str) {
            return 'nc-' + str;
        };
        
        this.encode = function NCName_encode (str) {
            str = str.replace( NOT_NC_CHARS_RE, encodeSymbol);
            str = str.replace(/^([.0-9]|$)/, addPrefix);
            return str;
        };
        
        this.decode = function NCName_decode (str) {
            str = str.replace(/^nc-/, '')
            str = str.replace(/(\.[0-9A-F]{4})/g, decodeSymbol)
            return str;
        };
    }
    
    const typeOf = function JSONvsXML__typeOf (val) {
        if (val === null) return "null";
        if (Object.prototype.toString.call(val) === '[object Array]') return "array";
        return typeof val;
    }
    
    this.xml4json = function JSONvsXML_xml4json ( json ){
        if (typeof json !== "object" ) return <>{json}</>;
        
        let parent = <></>
        for(var key in json){
            if (!json.hasOwnProperty(key)) continue;
            
            let val = json[key];
            let type = typeOf(val);
            let name = NCName.encode(key);
            
            let node = <{name} key={key} type={type} />;
            node.appendChild(JSONvsXML_xml4json(val));
            parent+=(node);
        }
        return parent;
    };
    
    this.json2domFragment = function JSONvsXML_json2dom(json, destDocument) {
        if (!(destDocument instanceof Components.interfaces.nsIDOMDocument))
            throw new TypeError("Second argument must be of type nsIDOMDocument. Got " + destDocument);
        
        let fragment = destDocument.createDocumentFragment();
        
        if (typeof json !== "object" ) {
            fragment.appendChild(destDocument.createTextNode(json));
            return fragment;
        }
        
        for (let key in json) {
            if (!json.hasOwnProperty(key)) continue;
            
            let val = json[key];
            let type = typeOf(val);
            let name = NCName.encode(key);
            
            let node = destDocument.createElement(name);
            node.setAttribute("key", key);
            node.setAttribute("type", type);
            
            node.appendChild(JSONvsXML_json2dom(val, destDocument));
            
            fragment.appendChild(node);
        }
        
        return fragment;
    }
};
